==> dart_icalendar-1.0.11.tar.gz: lib/models/x_date_model.dart <==
import 'dart:convert';

import 'package:dart_icalendar/models/time_zone_model.dart';
import 'package:intl/intl.dart';

class XDate {
  DateTime date;
  TimeZone timeZone;

  XDate({
    required this.date,
  }) : timeZone = TimeZone(tzId: date.timeZoneName, transitions: []);

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('EXDATE');
    if (timeZone.isUtc) {
      buffer.write('Z');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    } else {
      buffer.write(';TZID=${timeZone.tzId}');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    }
    return buffer.toString();
  }

  factory XDate.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    String? dateStr;

    for (var i = 0; i < lines.length; i++) {
      final line = lines[i];
      if (line.startsWith('EXDATE')) {
        final dateAndTimeZoneStr = line.split(';').last.trim();
        var timeZoneStr = dateAndTimeZoneStr.split(':').length > 1
            ? dateAndTimeZoneStr.split(':').first
            : null;
        timeZoneStr = timeZoneStr?.replaceAll('TZID=', '');
        dateStr = dateAndTimeZoneStr.replaceAll(timeZoneStr ?? '', '').trim();
        dateStr = dateStr.replaceAll('TZID=', '');
        dateStr = dateStr.replaceAll(':', '');
        break;
      }
    }

    if (dateStr == null) {
      throw FormatException('Invalid iCal string. DTSTART not found.');
    }

    final date = DateTime.parse(dateStr);
    return XDate(date: date);
  }

  XDate copyWith({
    DateTime? date,
    TimeZone? timeZone,
  }) {
    return XDate(
      date: date ?? this.date,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'date': date.millisecondsSinceEpoch,
      'time_zone': timeZone.toMap(),
    };
  }

  factory XDate.fromMap(Map<String, dynamic> map) {
    return XDate(
      date: DateTime.fromMillisecondsSinceEpoch(map['date']),
    );
  }

  String toJson() => json.encode(toMap());

  factory XDate.fromJson(String source) => XDate.fromMap(json.decode(source));

  @override
  String toString() => 'XDate(date: $date, timeZone: $timeZone)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is XDate && other.date == date && other.timeZone == timeZone;
  }

  @override
  int get hashCode => date.hashCode ^ timeZone.hashCode;
}

==> dart_icalendar-1.0.14.tar.gz: lib/models/x_date_model.dart <==
import 'dart:convert';

import 'package:dart_icalendar/models/time_zone_model.dart';
import 'package:intl/intl.dart';

class XDate {
  DateTime date;
  TimeZone timeZone;

  XDate({
    required this.date,
  }) : timeZone = TimeZone(tzId: date.timeZoneName, transitions: []);

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('EXDATE');
    if (timeZone.isUtc) {
      buffer.write('Z');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    } else {
      buffer.write(';TZID=${timeZone.tzId}');
      buffer.write(':');
      buffer.write(DateFormat('yyyyMMdd\'T\'HHmmss').format(date));
    }
    return buffer.toString();
  }

  factory XDate.fromICalString(String icalString) {
    final lines = icalString.split('\n');
    String? dateStr;

    for (var i = 0; i < lines.length; i++) {
      final line = lines[i];
      if (line.startsWith('EXDATE')) {
        final dateAndTimeZoneStr = line.split(';').last.trim();
        var timeZoneStr = dateAndTimeZoneStr.split(':').length > 1
            ? dateAndTimeZoneStr.split(':').first
            : null;
        timeZoneStr = timeZoneStr?.replaceAll('TZID=', '');
        dateStr = dateAndTimeZoneStr.replaceAll(timeZoneStr ?? '', '').trim();
        dateStr = dateStr.replaceAll('TZID=', '');
        dateStr = dateStr.replaceAll(':', '');
        break;
      }
    }

    if (dateStr == null) {
      throw FormatException('Invalid iCal string. DTSTART not found.');
    }

    final date = DateTime.parse(dateStr);
    return XDate(date: date);
  }

  XDate copyWith({
    DateTime? date,
    TimeZone? timeZone,
  }) {
    return XDate(
      date: date ?? this.date,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'date': date.millisecondsSinceEpoch,
      'time_zone': timeZone.toMap(),
    };
  }

  factory XDate.fromMap(Map<String, dynamic> map) {
    return XDate(
      date: DateTime.fromMillisecondsSinceEpoch(
          int.parse(map['date'].toString())),
    );
  }

  String toJson() => json.encode(toMap());

  factory XDate.fromJson(String source) => XDate.fromMap(json.decode(source));

  @override
  String toString() => 'XDate(date: $date, timeZone: $timeZone)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is XDate && other.date == date && other.timeZone == timeZone;
  }

  @override
  int get hashCode => date.hashCode ^ timeZone.hashCode;
}
